import re
import csv

def convert_srt_time_to_hhmmssff(time_str):
    """Convert SRT timecode to hh:mm:ss:ff format."""
    hours, minutes, seconds_milliseconds = time_str.split(':')
    seconds, milliseconds = seconds_milliseconds.split(',')
    frame = int(round(float(milliseconds) / 1000 * 30))  # Assuming 30 fps
    return f"{hours}:{minutes}:{seconds}:{frame:02d}"

def parse_srt_file(srt_filename, output_filename):
    """Parse SRT file and write converted data to CSV file."""
    last_known_role = ""
    entries = []
    pattern = re.compile(r'(\d\d:\d\d:\d\d,\d\d\d) --> (\d\d:\d\d:\d\d,\d\d\d)')
    
    with open(srt_filename, 'r', encoding='utf-8') as file:
        lines = file.readlines()
        
        i = 0
        index_number = 1  # Initialize index_number with 1
        role_reset_needed = False  # Flag to indicate if role needs to be reset
        
        while i < len(lines):
            if lines[i].strip().isdigit():  # Skip index number
                index_number = int(lines[i].strip())  # Assign index_number
                i += 1
            
            if '-->' in lines[i]:  # Time code line
                match = pattern.match(lines[i])
                if match:
                    start_time, end_time = match.groups()
                    start_time_converted = convert_srt_time_to_hhmmssff(start_time)
                    end_time_converted = convert_srt_time_to_hhmmssff(end_time)
                    
                    i += 1
                    text = ''
                    while i < len(lines) and lines[i].strip() != '':
                        text += lines[i].strip()
                        i += 1
                    
                    # Extract comment and remove brackets
                    comment_match = re.search(r'【.*?】', text)
                    comment = comment_match.group(0).strip('【】') if comment_match else ''
                    text = re.sub(r'【.*?】', '', text).strip()
                    
                    # Determine role name and dialog based on context
                    if comment_match:
                        role_name = ''  # Set role name to empty if there is a comment
                        dialog = text.strip()  # Assign dialog directly from text
                        role_reset_needed = True  # Reset role after this entry
                    else:
                        parts = text.split(':', 1)
                        if len(parts) == 2:
                            potential_role_name = parts[0].strip()
                            potential_dialog = parts[1].strip()
                            
                            # Check if the potential role name is short enough to be a role name
                            if len(potential_role_name) <= 5:
                                role_name = potential_role_name
                                dialog = potential_dialog
                                last_known_role = role_name  # Update last_known_role
                                role_reset_needed = False  # Reset flag since we have a new role
                            else:
                                role_name = last_known_role
                                dialog = text.strip()
                        else:
                            role_name = last_known_role
                            dialog = text.strip()
                    
                    # Use the previously assigned index_number
                    id_value = index_number
                    
                    entries.append({
                        "轨道名称": "标记",
                        "时码输入": start_time_converted,
                        "时码输出": end_time_converted,
                        "描述": '',
                        "长度": '',
                        "ID": id_value,
                        "注释": comment,
                        "录音完成": '',
                        "角色名字": role_name if not role_reset_needed else '',
                        "对话": dialog,
                        "对话翻译 1": ''
                    })
                i += 1
            else:
                i += 1
    
    with open(output_filename, 'w', newline='', encoding='utf-8') as csvfile:
        fieldnames = ["轨道名称", "时码输入", "时码输出", "描述", "长度", "ID", "注释", "录音完成", "角色名字", "对话", "对话翻译 1"]
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        
        writer.writeheader()
        for entry in entries:
            writer.writerow(entry)

# 指定输入和输出文件的位置
input_filename = 'path/to/your_srt_file.srt'  # 替换为你的SRT文件的实际路径
output_filename = 'path/to/converted.csv'  # 替换为你想要保存CSV文件的路径

# 调用函数
parse_srt_file(input_filename, output_filename)
print(f"转换完成，CSV文件已保存为 {output_filename}")